# -*- coding: utf-8 -*-

"""
Copyright (c) 2012, Red Hat, Inc

kitchen is free software; you can redistribute it and/or modify it under the
terms of the GNU Lesser General Public License as published by the Free
Software Foundation; either version 2.1 of the License, or (at your option)
any later version.
 
kitchen is distributed in the hope that it will be useful, but WITHOUT ANY
WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for
more details.
 
You should have received a copy of the GNU Lesser General Public License
along with kitchen; if not, see <http://www.gnu.org/licenses/>

Authors:
   Toshio Kuratomi <toshio@fedoraproject.org>
   Pierre-Yves Chibon <pingou@pingoured.fr>
"""


import ConfigParser
import glob
import os
import shutil
from kitchen.pycompat27 import subprocess
from kitchen.exceptions import KitchenError


__version__ = '0.1.0'


class MsgFmt(object):
    def run(self, args):
        cmd = subprocess.Popen(args, shell=False)
        cmd.wait()


class Utils(object):
    """ This utils class provides way to deal with catalogs for releases.
    """

    def __init__(catalog='auto', config=None, outputdir='locale'):
        """ Constructor.
        Set the attributes and their default values.
        :kwarg catalog, the name of the catalog to use. It can be: 
            msgfmt, babel or auto.
        :kwarg config, a potential transifex configuration file to use.
        :kwarg outputdir, the name of the output directory
        """
        self.catalog = catalog
        self.config = config
        self.outputdir = outputdir

    def _get_locale(self, file_filter, source_file):
        """ Retrieve a given locale.
        :kwarg file_filter, filter to recognize .po file
        (example if your .po files are under the po folder: po/<lang>.po)
        :kwarg source_file, the name of the source file for the
        translation (for example if it is under the po folder:
        po/kitchen.pot)
        """
        glob_pattern = file_filter.replace('<lang>', '*')
        pot = os.path.basename(source_file)
        if pot.endswith('.pot'):
            pot = pot[:-4]
        arg_values = {'domain': pot}
        for po_file in glob.glob(glob_pattern):
            file_pattern = os.path.basename(po_file)
            lang = file_pattern.replace('.po','')
            os.makedirs(os.path.join(self.outputdir, lang, 'LC_MESSAGES'))
            arg_values['pofile'] = po_file
            arg_values['lang'] = lang
            compile_args = args % arg_values
            compile_args = compile_args.split(' ')
            cmd.run(compile_args)

    def main(self, file_filter=None, source_file=None):
        """ Main function.
        :kwarg file_filter, filter to recognize .po file
        (example if your .po files are under the po folder: po/<lang>.po)
        :kwarg source_file, the name of the source file for the
        translation (for example if it is under the po folder:
        po/kitchen.pot)
        """
        try:
            shutil.rmtree(self.outputdir)
        except OSError, e:
            # If the error is that the directory does not exist, we're
            # okay. We're deleting it here, afterall
            if e.errno != 2:
                raise

        cmd, args = self.setup_message_compiler()

        if file_filter and source_file:
            self._get_locale(file_filter, source_file)
        # Get the directory with message catalogs
        # Reuse transifex's config file first as it will know this
        elif self.config:
            cfg = ConfigParser.SafeConfigParser()
            cfg.read(self.config)

            for section in [s for s in cfg.sections() if s != 'main']:
                try:
                    file_filter = cfg.get(section, 'file_filter')
                    source_file = cfg.get(section, 'source_file')
                except ConfigParser.NoOptionError:
                    continue
                self._get_locale(file_filter, source_file, cmd)

    def setup_message_compiler(self):
        if self.catalog == 'auto':
            # Look for msgfmt
            try:
                subprocess.Popen(['msgfmt', '-h'], stdout=subprocess.PIPE)
            except OSError:
                import babel.messages.frontend

                return (babel.messages.frontend.CommandLineInterface(),
                    'pybabel compile -D %(domain)s -d locale -i ' \
                    '%(pofile)s -l %(lang)s'
                    )
            else:
                return (MsgFmt(), 
                'msgfmt -c -o locale/%(lang)s/LC_MESSAGES/%(domain)s.mo' \
                ' %(pofile)s')
        elif self.catalog == 'msgfmt':
            return (MsgFmt(), 
                'msgfmt -c -o locale/%(lang)s/LC_MESSAGES/%(domain)s.mo' \
                ' %(pofile)s')
        elif self.catalog == 'babel':
            return (babel.messages.frontend.CommandLineInterface(),
                    'pybabel compile -D %(domain)s -d locale -i ' \
                    '%(pofile)s -l %(lang)s'
                    )
        else:
            raise KitchenError('Unabled to find the catalog used.')
